@@ -1,3 +1,4 @@
+/* $Id: ClpSimplex.hpp 1532 2010-03-23 14:36:16Z forrest $ */
 // Copyright (C) 2002, International Business Machines
 // Corporation and others.  All Rights Reserved.
 
@@ -21,7 +22,7 @@ class ClpFactorization;
 class CoinIndexedVector;
 class ClpNonLinearCost;
 class ClpNodeStuff;
-class CoinModel;
+class CoinStructuredModel;
 class OsiClpSolverInterface;
 class CoinWarmStartBasis;
 class ClpDisasterHandler;
@@ -64,9 +65,9 @@ public:
   // For Dual
   enum FakeBound {
     noFake = 0x00,
-    bothFake = 0x01,
+    lowerFake = 0x01,
     upperFake = 0x02,
-    lowerFake = 0x03
+    bothFake = 0x03
   };
 
   /**@name Constructors and destructor and copy */
@@ -120,25 +121,6 @@ public:
       2 as 1 but give a bit extra if bigger needed
   */
   void setPersistenceFlag(int value);
-  /**
-     If you are re-using the same matrix again and again then the setup time
-     to do scaling may be significant.  Also you may not want to initialize all values
-     or return all values (especially if infeasible).  While an auxiliary model exists
-     it will be faster.  If options -1 then model is switched off.  Otherwise switched on
-     with following options.
-     1 - rhs is constant
-     2 - bounds are constant
-     4 - objective is constant
-     8 - solution in by basis and no djs etc in
-     16 - no duals out (but reduced costs)
-     32 - no output if infeasible
-  */
-  void auxiliaryModel(int options);
-  /// Switch off e.g. if people using presolve
-  void deleteAuxiliaryModel();
-  /// See if we have auxiliary model
-  inline bool usingAuxiliaryModel() const
-  { return auxiliaryModel_!=NULL;}
   /// Save a copy of model with certain state - normally without cuts
   void makeBaseModel();
   /// Switch off base model
@@ -290,6 +272,16 @@ public:
   /** Solves non-linear using reduced gradient.  Phase = 0 get feasible,
       =1 use solution */
   int reducedGradient(int phase=0);
+  /// Solve using structure of model and maybe in parallel
+  int solve(CoinStructuredModel * model);
+  /** This loads a model from a CoinStructuredModel object - returns number of errors.
+      If originalOrder then keep to order stored in blocks,
+      otherwise first column/rows correspond to first block - etc.
+      If keepSolution true and size is same as current then
+      keeps current status and solution
+  */
+  int loadProblem (  CoinStructuredModel & modelObject,
+		     bool originalOrder=true,bool keepSolution=false);
   /**
      When scaling is on it is possible that the scaled problem
      is feasible but the unscaled is not.  Clp returns a secondary
@@ -370,6 +362,8 @@ public:
   CoinWarmStartBasis * getBasis() const;
   /// Passes in factorization
   void setFactorization( ClpFactorization & factorization);
+  // Swaps factorization
+  ClpFactorization * swapFactorization( ClpFactorization * factorization);
   /// Copies in factorization to existing one
   void copyFactorization( ClpFactorization & factorization);
   /** Tightens primal bounds to make dual faster.  Unless
@@ -537,6 +531,8 @@ public:
   /// Set algorithm
   inline void setAlgorithm(int value)
   {algorithm_=value; } 
+  /// Return true if the objective limit test can be relied upon
+  bool isObjectiveLimitTestValid() const ;
   /// Sum of dual infeasibilities
   inline double sumDualInfeasibilities() const 
           { return sumDualInfeasibilities_;} 
@@ -588,7 +584,7 @@ public:
       then status used - so all nonbasic variables set to
       indicated bound and if any values changed (or ==2)  basic values re-computed.
   */
-  void checkSolution(int setToBounds=false);
+  void checkSolution(int setToBounds=0);
   /** Just check solution (for internal use) - sets sum of
       infeasibilities etc. */
   void checkSolutionInternal();
@@ -598,6 +594,9 @@ public:
   /// Useful column length arrays (0,1,2,3,4,5)
   inline CoinIndexedVector * columnArray(int index) const
   { return columnArray_[index];}
+  /// Sets firstFree_
+  inline void setFirstFree(int value)
+  { firstFree_=value;}
   //@}
 
   /******************** End of most useful part **************/
@@ -621,8 +620,11 @@ public:
   int createPiecewiseLinearCosts(const int * starts,
 		   const double * lower, const double * gradient);
   /// dual row pivot choice
-  ClpDualRowPivot * dualRowPivot() const
+  inline ClpDualRowPivot * dualRowPivot() const
   { return dualRowPivot_;}
+  /// primal column pivot choice
+  inline ClpPrimalColumnPivot * primalColumnPivot() const
+  { return primalColumnPivot_;}
   /// Returns true if model looks OK
   inline bool goodAccuracy() const
   { return (largestPrimalError_<1.0e-7&&largestDualError_<1.0e-7);}
@@ -694,6 +696,15 @@ protected:
   void checkDualSolution();
   /** This sets sum and number of infeasibilities (Dual and Primal) */
   void checkBothSolutions();
+  /**  If input negative scales objective so maximum <= -value
+       and returns scale factor used.  If positive unscales and also
+       redoes dual stuff
+  */
+  double scaleObjective(double value);
+  /// Solve using Dantzig-Wolfe decomposition and maybe in parallel
+  int solveDW(CoinStructuredModel * model);
+  /// Solve using Benders decomposition and maybe in parallel
+  int solveBenders(CoinStructuredModel * model);
 public:
   /** For advanced use.  When doing iterative solves things can get
       nasty so on values pass if incoming solution has largest
@@ -717,9 +728,16 @@ public:
   inline void setAlphaAccuracy(double value)
           { alphaAccuracy_ = value;} 
 public:
-  /// Disaster handler
+   /// Objective value
+   //inline double objectiveValue() const {
+  //return (objectiveValue_-bestPossibleImprovement_)*optimizationDirection_ - dblParam_[ClpObjOffset];
+  //}
+  /// Set disaster handler
   inline void setDisasterHandler(ClpDisasterHandler * handler)
   { disasterArea_= handler;}
+  /// Get disaster handler
+  inline ClpDisasterHandler * disasterHandler() const
+  { return disasterArea_;}
   /// Large bound value (for complementarity etc)
   inline double largeValue() const 
           { return largeValue_;} 
@@ -736,6 +754,12 @@ public:
   /// Largest error on basic duals
   inline void setLargestDualError(double value)
           { largestDualError_=value;} 
+  /// Get zero tolerance
+  inline double zeroTolerance() const 
+  { return zeroTolerance_;/*factorization_->zeroTolerance();*/} 
+  /// Set zero tolerance
+  inline void setZeroTolerance( double value)
+  { zeroTolerance_ = value;}
   /// Basic variables pivoting on which rows
   inline int * pivotVariable() const
           { return pivotVariable_;}
@@ -837,11 +861,11 @@ public:
   { return cost_;}
   inline Status getStatus(int sequence) const
   {return static_cast<Status> (status_[sequence]&7);}
-  inline void setStatus(int sequence, Status status)
+  inline void setStatus(int sequence, Status newstatus)
   {
     unsigned char & st_byte = status_[sequence];
     st_byte = static_cast<unsigned char>(st_byte & ~7);
-    st_byte = static_cast<unsigned char>(st_byte | status);
+    st_byte = static_cast<unsigned char>(st_byte | newstatus);
   }
   /// Start or reset using maximumRows_ and Columns_ - true if change
   bool startPermanentArrays();
@@ -874,6 +898,18 @@ public:
   /// Value of Out variable
   inline double valueOut() const
   { return valueOut_;}
+  /// Set value of out variable
+  inline void setValueOut(double value)
+  { valueOut_=value;}
+  /// Set lower of out variable
+  inline void setLowerOut(double value)
+  { lowerOut_=value;}
+  /// Set upper of out variable
+  inline void setUpperOut(double value)
+  { upperOut_=value;}
+  /// Set theta of out variable
+  inline void setTheta(double value)
+  { theta_=value;}
   /// Returns 1 if sequence indicates column
   inline int isColumn(int sequence) const
   { return sequence<numberColumns_ ? 1 : 0;}
@@ -916,6 +952,10 @@ public:
   /// Theta (pivot change)
   inline double theta() const
   { return theta_;}
+  /** Best possible improvement using djs (primal) or 
+      obj change by flipping bounds to make dual feasible (dual) */
+  inline double bestPossibleImprovement() const
+  { return bestPossibleImprovement_;}
   /// Return pointer to details of costs
   inline ClpNonLinearCost * nonLinearCost() const
   { return nonLinearCost_;}
@@ -924,6 +964,12 @@ public:
       2 bit - if presolved problem infeasible return
       4 bit - keep arrays like upper_ around
       8 bit - if factorization kept can still declare optimal at once
+      16 bit - if checking replaceColumn accuracy before updating
+      32 bit - say optimal if primal feasible!
+      64 bit - give up easily in dual (and say infeasible)
+      128 bit - no objective, 0-1 and in B&B
+      256 bit - in primal from dual or vice versa
+      512 bit - alternative use of solveType_
   */
   inline int moreSpecialOptions() const
   { return moreSpecialOptions_;}
@@ -931,6 +977,13 @@ public:
       1 bit - if presolve says infeasible in ClpSolve return
       2 bit - if presolved problem infeasible return
       4 bit - keep arrays like upper_ around
+      8 bit - no free or superBasic variables
+      16 bit - if checking replaceColumn accuracy before updating
+      32 bit - say optimal if primal feasible!
+      64 bit - give up easily in dual (and say infeasible)
+      128 bit - no objective, 0-1 and in B&B
+      256 bit - in primal from dual or vice versa
+      512 bit - alternative use of solveType_
   */
   inline void setMoreSpecialOptions(int value)
   { moreSpecialOptions_ = value;}
@@ -945,19 +998,19 @@ public:
   }
   inline FakeBound getFakeBound(int sequence) const
   {return static_cast<FakeBound> ((status_[sequence]>>3)&3);}
-  inline void setRowStatus(int sequence, Status status)
+  inline void setRowStatus(int sequence, Status newstatus)
   {
     unsigned char & st_byte = status_[sequence+numberColumns_];
     st_byte = static_cast<unsigned char>(st_byte & ~7);
-    st_byte = static_cast<unsigned char>(st_byte | status);
+    st_byte = static_cast<unsigned char>(st_byte | newstatus);
   }
   inline Status getRowStatus(int sequence) const
   {return static_cast<Status> (status_[sequence+numberColumns_]&7);}
-  inline void setColumnStatus(int sequence, Status status)
+  inline void setColumnStatus(int sequence, Status newstatus)
   {
     unsigned char & st_byte = status_[sequence];
     st_byte = static_cast<unsigned char>(st_byte & ~7);
-    st_byte = static_cast<unsigned char>(st_byte | status);
+    st_byte = static_cast<unsigned char>(st_byte | newstatus);
   }
   inline Status getColumnStatus(int sequence) const
   {return static_cast<Status> (status_[sequence]&7);}
@@ -998,7 +1051,7 @@ public:
   {return lastBadIteration_;}
   /// Progress flag - at present 0 bit says artificials out
   inline int progressFlag() const
-  {return progressFlag_;}
+  {return (progressFlag_&3);}
   /// Force re-factorization early 
   inline void forceFactorization(int value)
   { forceFactorization_ = value;}
@@ -1098,8 +1151,8 @@ public:
 
       /** Set a single column lower and upper bound */
       inline void setColBounds( int elementIndex,
-	double lower, double upper )
-       { setColumnBounds(elementIndex, lower, upper);}
+	double newlower, double newupper )
+       { setColumnBounds(elementIndex, newlower, newupper);}
 
       /** Set the bounds on a number of columns simultaneously<br>
     	  @param indexFirst,indexLast pointers to the beginning and after the
@@ -1133,6 +1186,8 @@ public:
       void setRowSetBounds(const int* indexFirst,
     				   const int* indexLast,
     				   const double* boundList);
+  /// Resizes rim part of model 
+  void resize (int newNumberRows, int newNumberColumns);
     
     //@}
 
@@ -1145,26 +1200,23 @@ protected:
    single array is the owner of memory 
   */
   //@{
-  /// Worst column primal infeasibility
-  double columnPrimalInfeasibility_;
-  /// Worst row primal infeasibility
-  double rowPrimalInfeasibility_;
+  /** Best possible improvement using djs (primal) or 
+      obj change by flipping bounds to make dual feasible (dual) */
+  double bestPossibleImprovement_;
+  /// Zero tolerance
+  double zeroTolerance_;
   /// Sequence of worst (-1 if feasible)
   int columnPrimalSequence_;
   /// Sequence of worst (-1 if feasible)
   int rowPrimalSequence_;
-  /// Worst column dual infeasibility
-  double columnDualInfeasibility_;
-  /// Worst row dual infeasibility
-  double rowDualInfeasibility_;
+  /// "Best" objective value
+  double bestObjectiveValue_;
   /// More special options - see set for details
   int moreSpecialOptions_;
   /// Iteration when we entered dual or primal
   int baseIteration_;
   /// Primal tolerance needed to make dual feasible (<largeTolerance)
   double primalToleranceToGetOptimal_;
-  /// Remaining largest dual infeasibility
-  double remainingDualInfeasibility_;
   /// Large bound value (for complementarity etc)
   double largeValue_;
   /// Largest error on Ax-b
@@ -1257,8 +1309,6 @@ protected:
   double * rowActivityWork_;
   /// Column activities - working copy
   double * columnActivityWork_;
-  /// Auxiliary model
-  ClpSimplex * auxiliaryModel_;
   /// Number of dual infeasibilities
   int numberDualInfeasibilities_;
   /// Number of dual infeasibilities (without free)
@@ -1337,6 +1387,10 @@ protected:
   double allowedInfeasibility_;
   /// Automatic scaling of objective and rhs and bounds
   int automaticScale_;
+  /// Maximum perturbation array size (take out when code rewritten)
+  int maximumPerturbationSize_;
+  /// Perturbation array (maximumPerturbationSize_)
+  double * perturbationArray_;
   /// A copy of model with certain state - normally without cuts
   ClpSimplex * baseModel_;
   /// For dealing with all issues of cycling etc
