@@ -2,10 +2,10 @@
 /*                                                                           */
 /* This file is part of the SYMPHONY MILP Solver Framework.                  */
 /*                                                                           */
-/* SYMPHONY was jointly developed by Ted Ralphs (tkralphs@lehigh.edu) and    */
+/* SYMPHONY was jointly developed by Ted Ralphs (ted@lehigh.edu) and         */
 /* Laci Ladanyi (ladanyi@us.ibm.com).                                        */
 /*                                                                           */
-/* (c) Copyright 2000-2008 Ted Ralphs. All Rights Reserved.                  */
+/* (c) Copyright 2000-2010 Ted Ralphs. All Rights Reserved.                  */
 /*                                                                           */
 /* This software is licensed under the Common Public License. Please see     */
 /* accompanying file for terms.                                              */
@@ -25,7 +25,7 @@
 #define ERROR__NUMERICAL_INSTABILITY   -103
 #define ERROR__ILLEGAL_BRANCHING       -104
 #define ERROR__COMM_ERROR              -105
-
+#define ERROR__DUAL_INFEASIBLE         -106  
 /*---------------------------- type of the problem --------------------------*/
 #define ZERO_ONE_PROBLEM         0
 #define INTEGER_PROBLEM          1
@@ -49,6 +49,7 @@
 #define NODE_STATUS__PRUNED       4
 #define NODE_STATUS__INTERRUPTED  5
 #define NODE_STATUS__WARM_STARTED 6
+#define NODE_STATUS__WSPRUNED     7
 /*------------------------------ not_fixed stati ----------------------------*/
 #define NF_CHECK_ALL             0x00
 #define NF_CHECK_AFTER_LAST      0x01
@@ -82,6 +83,9 @@
 #define VBC_ACTIVE_NODE         3 /*White*/
 #define VBC_CAND_NODE           4 /*Light Red*/
 #define VBC_FEAS_SOL_FOUND      5 /*Blue*/
+#define VBC_PRUNED_INFEASIBLE   6 /*color ??, used only when vbc_emulation = 3*/
+#define VBC_PRUNED_FATHOMED     7 /*do*/
+#define VBC_IGNORE              8 /*do*/
 
 
 /*------------------ what to do with pruned nodes in the TM -----------------*/
@@ -95,21 +99,6 @@
 #define FULL_LOGGING             1
 #define VBC_TOOL                 2
 
-/*-------------------- emulation options for vbctool ------------------------*/
-#define NO_VBC_EMULATION         0
-#define VBC_EMULATION_FILE       1
-#define VBC_EMULATION_LIVE       2
-
-/*------------ search order options for multi-criteria problems -------------*/
-#define MC_FIFO                  0
-#define MC_LIFO                  1
-
-/*------------  warm_starting options for multi-criteria problems -------------*/
-#define MC_WS_UTOPIA_FIRST               0
-#define MC_WS_UTOPIA_BOTH_FIXED          1
-#define MC_WS_UTOPIA_BOTH                2
-#define MC_WS_BEST_CLOSE                 3
-
 /*****************************************************************************
  *****************************************************************************
  *************                                                      **********
@@ -191,6 +180,26 @@
 #define INTERNAL_CUT_GEN           -3
 #define EXTERNAL_CUT_GEN           -4
 
+
+/* --------------- type of cgl cut generator --------------------------------*/
+#define CGL_PROBING_GENERATOR       0
+#define CGL_KNAPSACK_GENERATOR      1
+#define CGL_CLIQUE_GENERATOR        2
+#define CGL_GOMORY_GENERATOR        3
+#define CGL_TWOMIR_GENERATOR        4
+#define CGL_FLOWCOVER_GENERATOR     5
+#define CGL_ODDHOLE_GENERATOR       6
+#define CGL_MIR_GENERATOR           7
+#define CGL_NUM_GENERATORS          7
+
+
+/* --------------- the status of cut generation on the chain ----------------*/
+#define CGL_CHAIN_START             0
+#define CGL_CHAIN_CONTINUE          1
+#define CGL_CHAIN_CHECK             2
+#define CGL_CHAIN_PAUSE             3  
+#define CGL_CHAIN_STOP              4
+
 /*---------------- what is the status of a particular constraint ------------*/
 #define VIOLATED_ROW                0
 #define TIGHT_ROW                   1
@@ -274,10 +283,15 @@
 #define DO_BRANCH                   0
 #define DO_NOT_BRANCH               1
 #define DO_NOT_BRANCH__FATHOMED     2
+#define DO_NOT_BRANCH__FEAS_SOL     3
 
 /*---------------- possible return values of branch() -----------------------*/
 #define NEW_NODE                     -1
-#define FATHOMED_NODE                -2
+#define FATHOMED_NODE                -2 
+#define FEAS_SOL_FOUND               -3
+/* asm4: added this return code for the case when the node can be pruned by
+ * branching */
+#define BRANCHING_INF_NODE           -4 
 
 /*------------- normal return value of various functions --------------------*/
 
@@ -380,12 +394,13 @@
 #define LP_SOL_WITHIN_LP         2
 /* no result */
 
+/*-------------------- emulation options for vbctool ------------------------*/
+#define NO_VBC_EMULATION         0
+#define VBC_EMULATION_FILE       1
+#define VBC_EMULATION_LIVE       2
+#define VBC_EMULATION_FILE_NEW   3
+
 /*------------------------ compare_candidates -------------------------------*/
-#define BIGGEST_DIFFERENCE_OBJ   0
-#define LOWEST_LOW_OBJ           1
-#define HIGHEST_LOW_OBJ          2
-#define LOWEST_HIGH_OBJ          3
-#define HIGHEST_HIGH_OBJ         4
 #ifdef COMPILE_FRAC_BRANCHING
 #define HIGHEST_LOW_FRAC         5
 #define LOWEST_LOW_FRAC          6
@@ -400,8 +415,6 @@
 #define BRANCH_ON_IT                              1
 
 /*--------------------------- select_child ----------------------------------*/
-#define PREFER_LOWER_OBJ_VALUE   0
-#define PREFER_HIGHER_OBJ_VALUE  1
 #ifdef COMPILE_FRAC_BRANCHING
 #define PREFER_MORE_FRACTIONAL   2
 #define PREFER_LESS_FRACTIONAL   3
@@ -426,18 +439,6 @@
 #define USER__CLOSE_TO_HALF_AND_EXPENSIVE  11
 #define USER__CLOSE_TO_ONE_AND_CHEAP       12
 
-/*-------------------- generate_cuts_in_lp defaults -------------------------*/
-#define GENERATE_CGL_CUTS                  20
-#define DO_NOT_GENERATE_CGL_CUTS           21
-
-/*-------------------- xxx_cuts_generation_levels ---------------------------*/
-#define DO_NOT_GENERATE        -1
-#define GENERATE_DEFAULT        0
-#define GENERATE_ONLY_IN_ROOT   1
-#define GENERATE_IF_IN_ROOT     2    /*default*/
-#define GENERATE_ALWAYS         3 
-#define GENERATE_PERIODICALLY   4
-
 /*****************************************************************************
  *****************************************************************************
  *************                                                      **********
@@ -451,19 +452,6 @@
 #define NEW_NODE__STARTED       -2
 #define NEW_NODE__ERROR         -3
 
-/*------------------------- node selection rules ----------------------------*/
-#define LOWEST_LP_FIRST       0
-#define HIGHEST_LP_FIRST      1
-#define BREADTH_FIRST_SEARCH  2
-#define DEPTH_FIRST_SEARCH    3
-#define BEST_FIRST_SEARCH     4
-#define DEPTH_FIRST_THEN_BEST_FIRST 5
-
-/*-------------------------- diving_strategy --------------------------------*/
-#define BEST_ESTIMATE         0
-#define COMP_BEST_K           1
-#define COMP_BEST_K_GAP       2
-
 /*****************************************************************************
  *****************************************************************************
  *************                                                      **********
@@ -490,8 +478,6 @@
 #define DELETE_DUPLICATES                 2
 #define DELETE_DUPLICATE_AND_INEFFECTIVE  2
 
-
-
 /*--------------- parameter values for restart/sens analysis ----------------*/
 
 #define NOTHING_CHANGED                   0
@@ -501,4 +487,74 @@
 #define COL_BOUNDS_CHANGED                4  
 #define OBJ_SENSE_CHANGED                 5
 #define RHS_SENSE_CHANGED                 6 
+#define COLS_ADDED                        7
+
+/*--------------- parameter values for restart/sens analysis ----------------*/
+#define DO_NOT_TRIM          0
+#define TRIM_LEVEL           1
+#define TRIM_INDEX           2
+#define ON_CRU_VARS          3
+
+
+/*--------------------- order type of a  matrix -----------------------------*/
+#define MAT_ROW_ORDERED         0
+#define MAT_COL_ORDERED         1
+
+/*---------------------- type of an implication -----------------------------*/
+#define IMP_ROW             0
+#define IMP_COL             1
+
+/*-----------------  return codes for presolve functions --------------------*/
+/* preprocessor exited without modifying the MIP in any way */
+#define PREP_UNMODIFIED     0
+/* preprocessor modified the MIP in some way */  
+#define PREP_MODIFIED       1
+/* preprocessor found the MIP infeasible */
+#define PREP_INFEAS         2
+/* preprocessor found the MIP unbounded */
+#define PREP_SOLVED         3
+/* preprocessor solved the MIP */
+#define PREP_UNBOUNDED      4
+#define PREP_NUMERIC_ERROR -1
+#define PREP_OTHER_ERROR   -2
+
+/*--------------- types used for collecting info about the problem ----*/
+
+/* used to define prob type and/or row type */
+#define CONTINUOUS_TYPE       0
+#define BINARY_TYPE           1
+#define INTEGER_TYPE          2
+//#define INTEGER_PROBLEM          1
+//#define MIXED_INTEGER_PROBLEM    2
+#define BIN_CONT_TYPE         3
+#define BIN_INT_TYPE          4
+#define INT_CONT_TYPE         5
+#define ALL_MIXED_TYPE        6
+
+#if 0
+/* row type */
+#define CONTINUOUS_ROW           0
+#define BINARY_ROW               1
+#define INTEGER_ROW              2
+#define BIN_CONT_ROW             3
+#define BIN_INT_ROW              4
+#define INT_CONT_ROW             5
+#define ALL_MIXED_ROW            6
+#endif 
+
+/* row bound type*/
+#define OPEN_ROW                 0
+#define ALL_BOUNDED_ROW          1
+#define MIXED_BOUNDED_ROW        2
+
+/* vec coefs type*/
+#define ALL_INTEGER_VEC          0
+#define ALL_BINARY_VEC           1
+#define FRACTIONAL_VEC           2
+
+/* vec sign type */
+#define MIXED_TYPE_VEC           0
+#define ALL_POS_VEC              1
+#define ALL_NEG_VEC              2
+
 #endif
